@@ -72,8 +72,8 @@ RS_XML(createAttributesList)(const char **atts, const xmlChar *encoding)
   PROTECT(attr_names = NEW_CHARACTER(n));
      ptr = atts;
      for(i=0; i < n; i++, ptr+=2) {
-      SET_STRING_ELT(attr_values, i, COPY_TO_USER_STRING(ptr[1]));
-      SET_STRING_ELT(attr_names, i,  COPY_TO_USER_STRING(ptr[0]));
+      SET_STRING_ELT(attr_values, i, ENC_COPY_TO_USER_STRING(ptr[1]));
+      SET_STRING_ELT(attr_names, i,  ENC_COPY_TO_USER_STRING(ptr[0]));
      }
     SET_NAMES(attr_values, attr_names);
   UNPROTECT(2);
@@ -88,7 +88,8 @@ RS_XML(Parse)(USER_OBJECT_ fileName, USER_OBJECT_ handlers, USER_OBJECT_ endElem
                 USER_OBJECT_ ignoreBlanks,  USER_OBJECT_ useTagName, USER_OBJECT_ asText,
                  USER_OBJECT_ trim, USER_OBJECT_ useExpat, USER_OBJECT_ stateObject,
                   USER_OBJECT_ replaceEntities, USER_OBJECT_ validate, USER_OBJECT_ saxVersion,
-   	           USER_OBJECT_ branches, USER_OBJECT_ useDotNames, USER_OBJECT_ errorFun)
+	      USER_OBJECT_ branches, USER_OBJECT_ useDotNames, USER_OBJECT_ errorFun,
+              USER_OBJECT_ manageMemory)
 {
 #ifdef LIBEXPAT
   FILE *file = NULL;
@@ -132,7 +133,7 @@ RS_XML(Parse)(USER_OBJECT_ fileName, USER_OBJECT_ handlers, USER_OBJECT_ endElem
     input = name;
   }
 
-  parserData = RS_XML(createParserData)(handlers);
+  parserData = RS_XML(createParserData)(handlers, manageMemory);
   parserData->endElementHandlers = endElementHandlers;
   parserData->branches         = branches;
   parserData->fileName         = name; 
@@ -211,14 +212,15 @@ RS_XML(entityDeclarationHandler)(void *userData, const XML_Char *entityName,
    xml_args[2] = systemId; xml_args[3] = publicId;
    xml_args[4] = notationName;
 
-  opArgs = NEW_LIST(num);
+  opArgs = PROTECT(NEW_LIST(num));
   for(i =0;i < num; i++) {
    SET_VECTOR_ELT(opArgs, i,  NEW_CHARACTER(1));
-   SET_STRING_ELT(VECTOR_ELT(opArgs, i), 0, COPY_TO_USER_STRING(xml_args[i] ? xml_args[i] :  "")); 
+   SET_STRING_ELT(VECTOR_ELT(opArgs, i), 0, ENC_COPY_TO_USER_STRING(xml_args[i] ? (const xmlChar *)xml_args[i] :  (const xmlChar *)"")); 
   }
 
   RS_XML(callUserFunction)(HANDLER_FUN_NAME(parserData, "entityDeclaration"), 
                            (const char*)NULL, parserData, opArgs);
+  UNPROTECT(1);
 }
 
 
@@ -237,7 +239,7 @@ RS_XML(startElement)(void *userData, const char *name, const char **atts)
 
   PROTECT(opArgs = NEW_LIST(2));
   SET_VECTOR_ELT(opArgs, 0, NEW_CHARACTER(1));
-  SET_STRING_ELT(VECTOR_ELT(opArgs, 0), 0, COPY_TO_USER_STRING(name)); 
+  SET_STRING_ELT(VECTOR_ELT(opArgs, 0), 0, ENC_COPY_TO_USER_STRING(name)); 
 
   /* Now convert the attributes list. */
    SET_VECTOR_ELT(opArgs, 1, RS_XML(createAttributesList)(atts, encoding));
@@ -254,7 +256,7 @@ RS_XML(commentHandler)(void *userData, const XML_Char *data)
 
   PROTECT(opArgs);
   SET_VECTOR_ELT(opArgs, 0, NEW_CHARACTER(1));
-     SET_STRING_ELT(VECTOR_ELT(opArgs, 0), 0, COPY_TO_USER_STRING(data));
+     SET_STRING_ELT(VECTOR_ELT(opArgs, 0), 0, ENC_COPY_TO_USER_STRING(data));
      RS_XML(callUserFunction)(HANDLER_FUN_NAME(rinfo, "comment"), 
 			      (const char *)NULL, ((RS_XMLParserData*)userData), opArgs);
   UNPROTECT(1);
@@ -292,7 +294,7 @@ void RS_XML(endElement)(void *userData, const char *name)
 
   PROTECT(opArgs = NEW_LIST(1));
   SET_VECTOR_ELT(opArgs, 0, NEW_CHARACTER(1));
-     SET_STRING_ELT(VECTOR_ELT(opArgs, 0), 0, COPY_TO_USER_STRING(name));
+     SET_STRING_ELT(VECTOR_ELT(opArgs, 0), 0, ENC_COPY_TO_USER_STRING(name));
 
   fun = findEndElementFun(name, rinfo);
   if(fun)  {
@@ -324,9 +326,9 @@ RS_XML(processingInstructionHandler)(void *userData, const XML_Char *target, con
 
  PROTECT(opArgs = NEW_LIST(2));
  SET_VECTOR_ELT(opArgs, 0, NEW_CHARACTER(1));
-   SET_STRING_ELT(VECTOR_ELT(opArgs, 0), 0, COPY_TO_USER_STRING(target));
+   SET_STRING_ELT(VECTOR_ELT(opArgs, 0), 0, ENC_COPY_TO_USER_STRING(target));
  SET_VECTOR_ELT(opArgs, 1, NEW_CHARACTER(1));
-   SET_STRING_ELT(VECTOR_ELT(opArgs, 1), 0, COPY_TO_USER_STRING(data));
+   SET_STRING_ELT(VECTOR_ELT(opArgs, 1), 0, ENC_COPY_TO_USER_STRING(data));
    RS_XML(callUserFunction)(HANDLER_FUN_NAME(parserData, "processingInstruction"), 
                              (const char *)NULL, (RS_XMLParserData*)userData, opArgs);
  UNPROTECT(1);
@@ -383,6 +385,7 @@ RS_XML(textHandler)(void *userData,  const XML_Char *s, int len)
  USER_OBJECT_ opArgs = NULL;
  RS_XMLParserData *parserData = (RS_XMLParserData*)userData; 
  DECL_ENCODING_FROM_EVENT_PARSER(parserData)
+ int nprot = 0;
 
    /* XXX Here is where we have to ignoreBlankLines and use the trim setting in parserData */
   if(parserData->current) {
@@ -401,7 +404,7 @@ RS_XML(textHandler)(void *userData,  const XML_Char *s, int len)
       tmpString = s;
 #endif
       if(newLen < 0)
-	tmp = strdup("");
+	tmp = (xmlChar *)strdup("");
       else {
          tmp = (xmlChar *) S_alloc(newLen + 2, sizeof(xmlChar));
          memcpy(tmp, tmpString, newLen); tmp[newLen] = '\0';
@@ -415,24 +418,24 @@ RS_XML(textHandler)(void *userData,  const XML_Char *s, int len)
      <abc/>
      <next>
      */
- if(s == (XML_Char*)NULL || s[0] == (XML_Char)NULL || len == 0 
+  if(s == (XML_Char*)NULL || s[0] == (XML_Char)0 || len == 0 
        || (len == 1 && ((const char *) s)[0] == '\n' && parserData->trim))
     return;
 
            /*XXX Deal with encoding, memory cleanup, 
              1 more than length so we can put a \0 on the end. */
-    tmp = tmpString = (char*)calloc(len+1, sizeof(char));
-    strncpy(tmpString, s, len);
+  tmp = tmpString = (char*)calloc(len+1, sizeof(char));
+  strncpy(tmpString, s, len);
  
-    if(parserData->trim) {
-      tmpString = trim(tmpString);
-      len = strlen(tmpString);
-    }
+  if(parserData->trim) {
+    tmpString = trim(tmpString);
+    len = strlen(tmpString);
+  }
 
   if(len > 0 || parserData->ignoreBlankLines == 0 ) {
-    PROTECT(opArgs = NEW_LIST(1));
-     SET_VECTOR_ELT(opArgs, 0, NEW_CHARACTER(1));
-     SET_STRING_ELT(VECTOR_ELT(opArgs, 0), 0, COPY_TO_USER_STRING(tmpString));
+    PROTECT(opArgs = NEW_LIST(1)); nprot++;
+    SET_VECTOR_ELT(opArgs, 0, NEW_CHARACTER(1));
+    SET_STRING_ELT(VECTOR_ELT(opArgs, 0), 0, ENC_COPY_TO_USER_STRING(tmpString));
   }
 
   free(tmp);
@@ -442,9 +445,9 @@ RS_XML(textHandler)(void *userData,  const XML_Char *s, int len)
      */
 
   if(opArgs != NULL) {
-      RS_XML(callUserFunction)(HANDLER_FUN_NAME(parserData, "text"), (const char *)NULL, ((RS_XMLParserData*) userData), opArgs);
-     UNPROTECT(1);
+    RS_XML(callUserFunction)(HANDLER_FUN_NAME(parserData, "text"), (const char *)NULL, ((RS_XMLParserData*) userData), opArgs);
   }
+  UNPROTECT(nprot);
 }
 
 
@@ -465,12 +468,13 @@ RS_XML(notStandAloneHandler)(void *userData)
   The caller should arrange to free it.
 */
 RS_XMLParserData *
-RS_XML(createParserData)(USER_OBJECT_ handlers) 
+RS_XML(createParserData)(USER_OBJECT_ handlers, USER_OBJECT_ finalize) 
 {
  RS_XMLParserData *parser = (RS_XMLParserData *) R_alloc(1, sizeof(RS_XMLParserData));
 
  memset(parser, '\0', sizeof(RS_XMLParserData));
  parser->methods = handlers;
+ parser->finalize = finalize;
 
 return(parser);
 }
@@ -487,7 +491,7 @@ RS_XML(callUserFunction)(const char *opName, const char *preferredName, RS_XMLPa
 {
   USER_OBJECT_ fun = NULL, val;
   USER_OBJECT_ _userObject = parserData->methods;
-  int general = 0;
+  // int general = 0;
 
   R_CHECK_INTERRUPTS
 
@@ -496,7 +500,7 @@ RS_XML(callUserFunction)(const char *opName, const char *preferredName, RS_XMLPa
   }
 
   if(fun == NULL) {
-    general = 1;
+    // general = 1;
     fun = RS_XML(findFunction)(opName, _userObject);
   }
 
